package com.codegeist.fisheye.internal.service;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import com.codegeist.fisheye.domain.PathInfo;

public class PathInfoHandler extends DefaultHandler implements Handler<PathInfo> {
	private StringBuilder buffer = new StringBuilder();
	private PathInfo result;
	private Boolean isFile;
	private Boolean isDir;
	private Boolean isHeadDeleted;
	private String pathName;
	private final String relativeTo;

	public PathInfoHandler(String relativeTo) {
		this.relativeTo = relativeTo;
	}
	
	@Override
	public void startDocument() throws SAXException {
		pathName = null;
		isDir = false;
		isFile = false;
		isHeadDeleted = false;
	}

	@Override
	public void startElement(String uri, String localName, String name, Attributes attributes) throws SAXException {
		buffer.setLength(0);
		if ("pathinfo".equals(localName)) {
			pathName = StringRegistry.internPath(attributes.getValue("name"));
			isFile = Boolean.valueOf(attributes.getValue("isFile"));
			isDir = Boolean.valueOf(attributes.getValue("isDir"));
			isHeadDeleted = Boolean.valueOf(attributes.getValue("isHeadDeleted"));
			
		}
	}
	
	@Override
	public void endElement(String uri, String localName, String name) throws SAXException {
		buffer.setLength(0);
	}
	
	
	@Override
	public void endDocument() throws SAXException {
		result = new PathInfo(relativeTo, pathName, isFile, isDir, isHeadDeleted);
	}
	
	@Override
	public void characters(char[] ch, int start, int length) throws SAXException {
		buffer.append(ch, start, length);
	}
	
	public PathInfo getResult() {
		return result;
	}
}
