package com.codegeist.fisheye.internal.service;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import com.codegeist.fisheye.domain.Revision;

public class RevisionHandler extends DefaultHandler implements Handler<Revision> {
	private StringBuilder buffer = new StringBuilder();
	private Revision result;
	private SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'");
	private String path;
	private String rev;
	private String state;
	private String author;
	private Date date;
	private int totalLines;
	private int linesAdded;
	private int linesRemoved;
	private String log;
	private String changesetId;
	private String ancestor;
	
	@Override
	public void startDocument() throws SAXException {
		path = null;
		rev = null;
		state = null;
		author = null;
		date = null;
		totalLines = 0;
		linesAdded = 0;
		linesRemoved = 0;
		log = null;
		changesetId = null;
		ancestor = null;
	}
	

	@Override
	public void startElement(String uri, String localName, String name, Attributes attributes) throws SAXException {
		buffer.setLength(0);
		if ("revision".equals(localName)) {
			path = StringRegistry.internPath(attributes.getValue("path"));
			rev = StringRegistry.internRev(attributes.getValue("rev"));
			author = StringRegistry.internAuthor(attributes.getValue("author"));
			date = parseDate(attributes.getValue("date"));
			state = attributes.getValue("state");
			totalLines = parseInt(attributes.getValue("totalLines"));
			linesAdded = parseInt(attributes.getValue("linesAdded"));
			linesRemoved = parseInt(attributes.getValue("linesRemoved"));
			changesetId = StringRegistry.internChangeSetId(attributes.getValue("csid"));
			ancestor = StringRegistry.internRev(attributes.getValue("ancestor"));
		}
	}
	
	@Override
	public void endElement(String uri, String localName, String name) throws SAXException {
		// TODO check that we see only one log element
		if ("log".equals(localName)) {
			log = buffer.toString();
		}
		buffer.setLength(0);
	}
	
	
	@Override
	public void endDocument() throws SAXException {
		result = new Revision(path, rev, author, date, state, totalLines, linesAdded, linesRemoved, log, changesetId, ancestor);
	}
	
	@Override
	public void characters(char[] ch, int start, int length) throws SAXException {
		buffer.append(ch, start, length);
	}
	
	public Revision getResult() {
		return result;
	}
	
	private int parseInt(String value) throws SAXException {
		try {
			return Integer.parseInt(value);
		} catch (NumberFormatException e) {
			// TODO fix this
			throw new SAXException("ZOMG");
		}
	}
	
	private Date parseDate(String value) throws SAXException {
		try {
			return format.parse(value);
		} catch (ParseException e) {
			// TODO fix this
			throw new SAXException("ZOMG");
		}
	}
}
